import { useParams } from "react-router-dom";
import { MoneyMgrWebRouteContainer } from "../widgets/MoneyMgrWebRouteContainer";
import { useAccounts } from "../hooks/AccountsListProvider";
import { NotFoundRoute } from "./NotFound";
import { AccountWidget } from "../widgets/AccountWidget";
import { AmountWidget } from "../widgets/AmountWidget";
import { Typography } from "@mui/material";
import { NewMovementWidget } from "../widgets/NewMovementWidget";
import { Movement, MovementApi } from "../api/MovementsApi";
import React from "react";
import { AsyncWidget } from "../widgets/AsyncWidget";
import { DataGrid, GridColDef } from "@mui/x-data-grid";
import { DateWidget } from "../widgets/DateWidget";
import { useAlert } from "../hooks/context_providers/AlertDialogProvider";

export function AccountRoute(): React.ReactElement {
  const { accountId } = useParams();

  const loadKey = React.useRef(0);

  const accounts = useAccounts();
  const account = accounts.get(Number(accountId));

  const [movements, setMovements] = React.useState<Movement[] | undefined>();

  const load = async () => {
    setMovements(undefined);
    setMovements(await MovementApi.GetAccountMovements(account!.id));
  };

  const reload = async () => {
    accounts.reloadBalances();
    setMovements(undefined);
    loadKey.current += 1;
  };

  if (account === null) return <NotFoundRoute />;

  return (
    <MoneyMgrWebRouteContainer
      label={
        <span style={{ display: "inline-flex", alignItems: "center" }}>
          <AccountWidget account={account} />
          &nbsp;
          <span style={{ display: "inline-flex", flexDirection: "column" }}>
            <span>{account.name}</span>
            <Typography component={"span"} variant="subtitle1">
              <AmountWidget amount={account.balance} />
            </Typography>
          </span>
        </span>
      }
    >
      <div style={{ display: "flex", flexDirection: "column", flex: 1 }}>
        <div style={{ flex: 1 }}>
          <AsyncWidget
            loadKey={`${account.id}-${loadKey.current}`}
            load={load}
            ready={movements !== null}
            errMsg="Failed to load the list of movements!"
            build={() => <MovementsTable movements={movements!} />}
          />
        </div>
        <NewMovementWidget account={account} onCreated={reload} />
      </div>
    </MoneyMgrWebRouteContainer>
  );
}

function MovementsTable(p: {
  movements: Movement[];
  needReload: () => {};
}): React.ReactElement {
  const alert = useAlert();

  const columns: GridColDef<(typeof p.movements)[number]>[] = [
    {
      field: "checked",
      headerName: "Checked",
      width: 50,
      type: "boolean",
      editable: true,
    },
    {
      field: "time",
      headerName: "Date",
      width: 98 + 80,
      editable: true,
      type: "dateTime",
      valueGetter(_, m) {
        return new Date(m.time * 1000);
      },
      valueSetter(v, row) {
        row.time = Math.floor(v.getTime() / 1000);
        return row;
      },
      renderCell: (params) => {
        return <DateWidget time={params.row.time} />;
      },
    },
    {
      field: "label",
      headerName: "Label",
      flex: 1,
      editable: true,
      type: "string",
    },
    {
      field: "amount",
      headerName: "Amount",
      width: 110,
      editable: true,
      type: "number",
      align: "left",
      headerAlign: "left",
      renderCell: (params) => {
        return <AmountWidget amount={params.row.amount} />;
      },
    },
    {
      field: "file",
      headerName: "File",
      // TODO
    },
  ];
  return (
    <DataGrid<Movement>
      columns={columns}
      rows={p.movements}
      autoPageSize
      checkboxSelection
      initialState={{
        sorting: {
          sortModel: [{ field: "time", sort: "desc" }],
        },
        columns: {
          columnVisibilityModel: {
            checked: false,
          },
        },
      }}
      processRowUpdate={async (n) => {
        try {
          return await MovementApi.Update(n);
        } catch (e) {
          console.error("Failed to update movement information!", e);
          alert(`Failed to update row! ${e}`);
          throw e;
        }
      }}
    />
  );
}
